use stanza::client::Stanza;
use tokio::sync::oneshot::{self};

#[derive(Debug)]
pub enum Error {
    AlreadyConnected,
    // TODO: change to Connecting(ConnectingError)
    Connection(ConnectionError),
    Presence(PresenceError),
    SetStatus(Reason),
    Roster(Reason),
    Stream(stanza::stream::Error),
    SendMessage(Reason),
    RecvMessage(RecvMessageError),
    AlreadyDisconnected,
    LostConnection,
    // TODO: should all cache update errors include the context?
    CacheUpdate(Reason),
    UnrecognizedContent(peanuts::element::Content),
    Iq(IqError),
    Cloned,
}

// TODO: this is horrifying, maybe just use tracing to forward error events???
impl Clone for Error {
    fn clone(&self) -> Self {
        Error::Cloned
    }
}

#[derive(Debug)]
pub enum PresenceError {
    Error(Reason),
    Unsupported,
    MissingFrom,
}

#[derive(Debug)]
pub enum IqError {
    NoMatchingId(String),
}

#[derive(Debug)]
pub enum RecvMessageError {
    MissingFrom,
}

#[derive(Debug, Clone)]
pub enum ConnectionError {
    ConnectionFailed(Reason),
    RosterRetreival(Reason),
    SendPresence(Reason),
    NoCachedStatus(Reason),
}

#[derive(Debug)]
pub struct RosterError(pub Reason);

impl From<RosterError> for Error {
    fn from(e: RosterError) -> Self {
        Self::Roster(e.0)
    }
}

impl From<RosterError> for ConnectionError {
    fn from(e: RosterError) -> Self {
        Self::RosterRetreival(e.0)
    }
}

pub struct StatusError(pub Reason);

impl From<StatusError> for Error {
    fn from(e: StatusError) -> Self {
        Error::SetStatus(e.0)
    }
}

impl From<StatusError> for ConnectionError {
    fn from(e: StatusError) -> Self {
        Self::SendPresence(e.0)
    }
}

#[derive(Debug)]
pub enum Reason {
    // TODO: organisastion of error into internal error thing
    Timeout,
    Stream(stanza::stream_error::Error),
    Stanza(Option<stanza::client::error::Error>),
    Jabber(jabber::Error),
    XML(peanuts::Error),
    SQL(sqlx::Error),
    // JID(jid::ParseError),
    LostConnection,
    OneshotRecv(oneshot::error::RecvError),
    UnexpectedStanza(Stanza),
    Disconnected,
    ChannelSend,
    Cloned,
}

// TODO: same here
impl Clone for Reason {
    fn clone(&self) -> Self {
        Reason::Cloned
    }
}

impl From<oneshot::error::RecvError> for Reason {
    fn from(e: oneshot::error::RecvError) -> Reason {
        Self::OneshotRecv(e)
    }
}

impl From<peanuts::Error> for Reason {
    fn from(e: peanuts::Error) -> Self {
        Self::XML(e)
    }
}

// impl From<jid::ParseError> for Reason {
//     fn from(e: jid::ParseError) -> Self {
//         Self::JID(e)
//     }
// }

impl From<sqlx::Error> for Reason {
    fn from(e: sqlx::Error) -> Self {
        Self::SQL(e)
    }
}

impl From<jabber::Error> for Reason {
    fn from(e: jabber::Error) -> Self {
        Self::Jabber(e)
    }
}
